package co.ab180.airbridge.flutter

import android.content.Intent
import android.net.Uri
import co.ab180.airbridge.Airbridge
import co.ab180.airbridge.AirbridgeCallback
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import java.util.*
import kotlin.collections.HashMap

import android.os.Handler
import android.os.Looper

class AttributionAPI(private val channel: EventChannel) : EventChannel.StreamHandler {
    companion object {
        val attributionTracker = AttributionTracker()
    }

    private var attributionListenerID: UUID? = null
    private val uiThreadHandler = Handler(Looper.getMainLooper())

    fun attachToChannel() {
        channel.setStreamHandler(this)
    }

    fun detachFromChannel() {
        channel.setStreamHandler(null)
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        val target = arguments as? String
        
        if (target == "attribution") {
            events?.let {
                attributionListenerID = attributionTracker.listen { result ->
                    uiThreadHandler.post {
                        events.success(result)
                    }
                }
            }
        }
    }

    override fun onCancel(arguments: Any?) {
        attributionListenerID?.let { id ->
            attributionTracker.stop(id)
        }
    }
}

class AttributionTracker {
    private var attribution: Map<String, String>? = null
    private var attributionListeners: MutableMap<UUID, (Map<String, String>) -> Unit> = HashMap()

    fun provideResult(result: Map<String, String>) {
        if (attribution == null) {
            attribution = result
        }

        attributionListeners.values.forEach { it(result) }
    }

    fun listen(listener: (Map<String, String>) -> Unit): UUID {
        if (attributionListeners.count() == 0) {
            attribution?.let { listener(it) }
        }

        val id = UUID.randomUUID()
        attributionListeners[id] = listener

        return id
    }

    fun stop(id: UUID) {
        attributionListeners.remove(id)
    }
}

fun AttributionAPI.Companion.fromPlugin(binding: FlutterPlugin.FlutterPluginBinding): AttributionAPI {
    return AttributionAPI(EventChannel(binding.binaryMessenger, "airbridge_flutter_sdk/event/attribution"))
}