//
//  AttributionAPI.swift
//  airbridge_flutter_sdk
//
//  Created by sangryel on 2024/02/28.
//

import Foundation
import AirBridge

class AttributionAPI: NSObject, FlutterStreamHandler {
    let channel: FlutterEventChannel
    var attributionListenerID: UUID? = nil
    
    init(channel: FlutterEventChannel) {
        self.channel = channel
    }
}

extension AttributionAPI {
    func attachToChannel() {
        channel.setStreamHandler(self)
    }
    
    func detachFromChannel() {
        channel.setStreamHandler(nil)
    }
}

extension AttributionAPI {
    private static var attributionTracker = AttributionTrakcer()

    static func provideResult(result: Dictionary<String, String>) {
        AttributionAPI.attributionTracker.provideResult(result: result)
    }
    
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        let target = arguments as? String
        
        if target == "attribution" {
            attributionListenerID = AttributionAPI.attributionTracker.listen { result in
                events(result)
            }
        }
        
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        if let attributionListenerID = attributionListenerID {
            AttributionAPI.attributionTracker.stop(id: attributionListenerID)
        }
        
        return nil
    }
}

fileprivate struct AttributionTrakcer {
    var attribution: Dictionary<String, String>? = nil
    private var attributionListeners: Dictionary<UUID, (Dictionary<String, String>) -> Void> = [:]
    
    mutating func provideResult(result: Dictionary<String, String>) {
        if attribution == nil {
            attribution = result
        }
        
        attributionListeners.values.forEach { $0(result) }
    }
    
    @discardableResult
    mutating func listen(listener: @escaping (Dictionary<String, String>) -> Void) -> UUID {
        if attributionListeners.count == 0 {
            if let attribution = attribution {
                listener(attribution)
            }
        }
        
        let id = UUID()
        attributionListeners[id] = listener
        
        return id
    }
    
    mutating func stop(id: UUID) {
        attributionListeners.removeValue(forKey: id)
    }
}

extension AttributionAPI {
    static func fromPlugin(_ registrar: FlutterPluginRegistrar) -> AttributionAPI {
        return AttributionAPI(channel: FlutterEventChannel(name: "airbridge_flutter_sdk/event/attribution",
                                                        binaryMessenger: registrar.messenger()))
    }
}
